@@ -1,12 +1,14 @@
-from neomodel import (StructuredNode, RelationshipTo, RelationshipFrom,
-        Relationship, StringProperty, IntegerProperty, StructuredRel, One)
+from pytest import raises
+
+from neomodel import (StructuredNode, RelationshipTo, RelationshipFrom, Relationship,
+                      StringProperty, IntegerProperty, StructuredRel, One)
 
 
-class Person(StructuredNode):
+class PersonWithRels(StructuredNode):
     name = StringProperty(unique_index=True)
     age = IntegerProperty(index=True)
     is_from = RelationshipTo('Country', 'IS_FROM')
-    knows = Relationship(u'Person', 'KNOWS')  # use unicode to check issue
+    knows = Relationship(u'PersonWithRels', 'KNOWS')  # use unicode to check issue
 
     @property
     def special_name(self):
@@ -18,11 +20,11 @@ class Person(StructuredNode):
 
 class Country(StructuredNode):
     code = StringProperty(unique_index=True)
-    inhabitant = RelationshipFrom(Person, 'IS_FROM')
-    president = RelationshipTo(Person, 'PRESIDENT', cardinality=One)
+    inhabitant = RelationshipFrom(PersonWithRels, 'IS_FROM')
+    president = RelationshipTo(PersonWithRels, 'PRESIDENT', cardinality=One)
 
 
-class SuperHero(Person):
+class SuperHero(PersonWithRels):
     power = StringProperty(index=True)
 
     def special_power(self):
@@ -30,32 +32,20 @@ class SuperHero(Person):
 
 
 def test_actions_on_deleted_node():
-    u = Person(name='Jim2', age=3).save()
+    u = PersonWithRels(name='Jim2', age=3).save()
     u.delete()
-    try:
+    with raises(ValueError):
         u.is_from.connect(None)
-    except ValueError:
-        assert True
-    else:
-        assert False
 
-    try:
+    with raises(ValueError):
         u.is_from.get()
-    except ValueError:
-        assert True
-    else:
-        assert False
 
-    try:
+    with raises(ValueError):
         u.save()
-    except ValueError:
-        assert True
-    else:
-        assert False
 
 
 def test_bidirectional_relationships():
-    u = Person(name='Jim', age=3).save()
+    u = PersonWithRels(name='Jim', age=3).save()
     assert u
 
     de = Country(code='DE').save()
@@ -84,8 +74,8 @@ def test_bidirectional_relationships():
 
 
 def test_either_direction_connect():
-    rey = Person(name='Rey', age=3).save()
-    sakis = Person(name='Sakis', age=3).save()
+    rey = PersonWithRels(name='Rey', age=3).save()
+    sakis = PersonWithRels(name='Sakis', age=3).save()
 
     rey.knows.connect(sakis)
     assert rey.knows.is_connected(sakis)
@@ -93,7 +83,7 @@ def test_either_direction_connect():
     sakis.knows.connect(rey)
 
     result, meta = sakis.cypher("""MATCH (us), (them)
-            WHERE id(us)={self} and id(them)={them}
+            WHERE id(us)=$self and id(them)=$them
             MATCH (us)-[r:KNOWS]-(them) RETURN COUNT(r)""",
             {'them': rey.id})
     assert int(result[0][0]) == 1
@@ -106,14 +96,14 @@ def test_either_direction_connect():
 
 
 def test_search_and_filter_and_exclude():
-    fred = Person(name='Fred', age=13).save()
+    fred = PersonWithRels(name='Fred', age=13).save()
     zz = Country(code='ZZ').save()
     zx = Country(code='ZX').save()
     zt = Country(code='ZY').save()
     fred.is_from.connect(zz)
     fred.is_from.connect(zx)
     fred.is_from.connect(zt)
-    result = fred.is_from.search(code='ZX')
+    result = fred.is_from.filter(code='ZX')
     assert result[0].code == 'ZX'
 
     result = fred.is_from.filter(code='ZX')
@@ -124,7 +114,7 @@ def test_search_and_filter_and_exclude():
 
 
 def test_custom_methods():
-    u = Person(name='Joe90', age=13).save()
+    u = PersonWithRels(name='Joe90', age=13).save()
     assert u.special_power() == "I have no powers"
     u = SuperHero(name='Joe91', age=13, power='xxx').save()
     assert u.special_power() == "I have powers"
@@ -132,10 +122,10 @@ def test_custom_methods():
 
 
 def test_valid_reconnection():
-    p = Person(name='ElPresidente', age=93).save()
+    p = PersonWithRels(name='ElPresidente', age=93).save()
     assert p
 
-    pp = Person(name='TheAdversary', age=33).save()
+    pp = PersonWithRels(name='TheAdversary', age=33).save()
     assert pp
 
     c = Country(code='CU').save()
@@ -153,8 +143,34 @@ def test_valid_reconnection():
     assert c.president.is_connected(pp)
 
 
+def test_valid_replace():
+    brady = PersonWithRels(name='Tom Brady', age=40).save()
+    assert brady
+
+    gronk = PersonWithRels(name='Rob Gronkowski', age=28).save()
+    assert gronk
+
+    colbert = PersonWithRels(name='Stephen Colbert', age=53).save()
+    assert colbert
+
+    hanks = PersonWithRels(name='Tom Hanks', age=61).save()
+    assert hanks
+
+    brady.knows.connect(gronk)
+    brady.knows.connect(colbert)
+    assert len(brady.knows) == 2
+    assert brady.knows.is_connected(gronk)
+    assert brady.knows.is_connected(colbert)
+
+    brady.knows.replace(hanks)
+    assert len(brady.knows) == 1
+    assert brady.knows.is_connected(hanks)
+    assert not brady.knows.is_connected(gronk)
+    assert not brady.knows.is_connected(colbert)
+
+
 def test_props_relationship():
-    u = Person(name='Mar', age=20).save()
+    u = PersonWithRels(name='Mar', age=20).save()
     assert u
 
     c = Country(code='AT').save()
@@ -163,10 +179,6 @@ def test_props_relationship():
     c2 = Country(code='LA').save()
     assert c2
 
-    try:
+    with raises(NotImplementedError):
         c.inhabitant.connect(u, properties={'city': 'Thessaloniki'})
-    except NotImplementedError:
-        assert True
-    else:
-        assert False
 
